      PROGRAM optmain
      IMPLICIT NONE

C******************************************************************
! Main optimization program: 
! Calls IFFCO and feeds in the propoer objective function FEVAL
!
C*****************************************************************
! Program created by Katie Kavanagh 9/16/02
! Last Modified: 2/10/03 Cleaned up the comments
C***************************************************************** 
      INTEGER I, DIM, MAXIT(2), RESTART, MAXCUTS, flag,WRIT
      INTEGER OPTION(30)
      INTEGER NLAY, NCOL, NROW
C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      EXTERNAL FEVAL, comminitIF, commexitIF
C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C!!!!! DIM= number of decsion variables
C Must be changed if the number of decsion variables changes
 
      PARAMETER(DIM = 18)

C !!!!! NROW, NLAY, NCOL must also be set in feval.f

      PARAMETER(NROW = 50)
      PARAMETER(NCOL = 50)
      PARAMETER(NLAY = 10)
C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DOUBLE PRECISION U(DIM), L(DIM), V(DIM)
      DOUBLE PRECISION FMIN, TERMTOL, FSCALE,MINH,MAXH,FV
      REAL*8 HEAD(NROW,NCOL,NLAY)
C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

C Set parameters for IFFCO

C Set upper and lower box constraints on the locations of the wells
      DO 10 , I=1,12
      U(I) = 800.00
      L(I) = 20.00
 10   CONTINUE

C Set the bounds on the pumping rates
      DO 20 , I=13,18
      U(I) = 0.002
      L(I) = -0.0064
 20   CONTINUE


C Other IFFCO parameters:

C It is best to set FSCALE to 20% higher than the function value
C at the intial iterate
      FSCALE = 2.0E5
      MINH = 0.002
      MAXH = 0.5
      MAXIT(1) = 100
      MAXIT(2) = 10000
      RESTART = 1
      WRIT = 4
      TERMTOL = 1.0D0
      MAXCUTS = 3
      OPTION(1) = 1
      OPTION(2) = 2
      OPTION(3) = 0
      OPTION(4) = 1     
      DO 30 , I=5,20
      OPTION(I) = 0
 30   CONTINUE
      OPTION(7) = 1
C**************************************************

C Set the initial iterate, Ie. Well locations and rates
      V(1) = 350.0
      V(2) = 725.0

      V(3) = 775.00
      V(4) = 775.00

      V(5) = 675.00
      V(6) = 675.00

      V(7) = 200.00
      V(8) = 200.00

      V(9) = 725.00
      V(10) = 300.00

      V(11) = 600.00
      V(12) = 600.00

C Initial Iterate 1 well rates
      DO 40 , I=13,18
       V(I) = -0.0064
 40    CONTINUE

C Initial Iterate 2 well rates
C      DO 40 , I=13,18
C       V(I) = -0.0054
C 40    CONTINUE



C*************************************************

C Call the IFFCO
      CALL COMMINITIF()
      CALL IFFCO(FEVAL,V,U,L,FSCALE,MINH,MAXH,DIM,MAXIT,
     +      RESTART,WRIT,TERMTOL,FMIN,MAXCUTS,OPTION)


C To evaluate the objective function at the intial iterate
C comment out the call to IFFCO and uncomment the call 
C to feval. This is useful when determining what to 
C set FSCALE to or to get a Matlab file for the initial
C well locations
C       CALL FEVAL(DIM,V,FV,FLAG)


C The following calls produce a matlab file with the
C head contours for the solution at the optmial point found by IFFCO.
C Commenting this out does not effect the optmization
     
      CALL POSTPROC(DIM,V,FV,FLAG,NROW,NCOL,NLAY,HEAD)
    

      CALL COMMEXITIF()

      END







