      SUBROUTINE feval(DIM,V,FV,FLAG)
      IMPLICIT NONE
      include "mpif.h"

C************************************************************************* 
!     SUBROUTINE TO EVALUATE THE OBJECTIVE FUNCTION FOR OPTIMIZATION 
!
!     THIS OBJECTIVE FUNCTION WILL:
!       1. SET THE VALUES FOR THE SIMULATION
!       2. CALL FLOW AND TRANSPORT SIMULATORS
!       3. EXCTRACT NECESSARY INFORMATION FROM OUTPUT FILES 
!          BY CALLING THE SUBROUTINES WRITEWEL AND READHEAD
!       4. COMPUTE THE COST BY CALLING THE SUBROUTINE COSTA(C,O)
C************************************************************************
!     Program created by Katie Kavanagh 9/11/02
!     Last modified: 4/4/03 
C***********************************************************************
!     INPUT:
!             DIM: DIMENSION OF THE PROBLEM
!               V: POINT TO EVALUATE THE FUNCTION AT 
!     OUTPUT:
!             FV: FUNCTION VALUE
!           FLAG: 0 FOR SUCCESS, >0 FOR FAILURE 
C************************************************************************
C Description of variables:
!
! NROW, NLAY, NCOL: number of rows, layers, columns    
! WELRATE: array of pumping rates
! COSTC, COST) capital and operation costs
! TOTALCOST = COSTC + COSTO
! HEAD array containing values of head, output from modflow
! DRWDOWN (not used yet) drawdown in each well
! STARTHEAD (not used yet) starting head in each well
C**********************************************************************
      CHARACTER*80 modflw96
    
      INTEGER DIM,I,J,K,FLAG,istat, mype,npes
      INTEGER NROW,NLAY,NCOL

      PARAMETER(NROW = 100)
      PARAMETER(NCOL = 100)
      PARAMETER(NLAY = 10)
      
      DOUBLE PRECISION MINRATE, QTMAX, QT
      DOUBLE PRECISION V(DIM),FV,WELRATE(DIM), COSTC,COSTO,TOTALCOST
      REAL*8 HEAD(NROW,NCOL,NLAY), DRWDOWN(DIM), STARTHEAD(DIM)
      INTEGER X(DIM), Y(DIM), Z(DIM)
      character*80 mfnfile    
  


      call MPI_COMM_rank(MPI_COMM_WORLD,mype,istat);
C Set values (MINRATE is the shut off pumping rate value)
       MINRATE = 0.000001
       QTMAX = -0.031998
       QT = 0.0
C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C Set your well locations 

       DO 90 , I=1,DIM/3
         Z(I) = 10
 90      CONTINUE


      
       X(1) = NINT((1000-V(2))/10.0) 
       Y(1) = NINT(V(1)/10.0)
  
       X(2) = NINT((1000-V(4))/10.0) 
       Y(2) = NINT(V(3)/10.0)

       X(3) = NINT((1000-V(6))/10.0) 
       Y(3) = NINT(V(5)/10.0)
  
       X(4) = NINT((1000-V(8))/10.0) 
       Y(4) = NINT(V(7)/10.0)  


C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
C Set the pumping rates at the 4 wells    
       DO 666 , I=1,DIM/3
       WELRATE(I) = V(I+8)
       IF(ABS(WELRATE(I)) .LT. MINRATE)THEN
       WELRATE(I) = 0.0
       ENDIF
 666   CONTINUE

C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C Set FLAG = 0 for success
      FLAG = 0
C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C Check that 2 wells do not occupy the same location
      call location_constraint(DIM/3,X,Y,FLAG)

C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C Check the constraints on the net pumping rate
      call linear_constraint(DIM/3,X,Y,Z,WELRATE,NLAY,
     & NROW, NCOL,FLAG)
C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C Compute the cost
      IF(FLAG .EQ. 0)THEN
C Set the .mfn file for modflow
      call writemfn(mype)
      if(mype<10)then
      write(mfnfile,'(a2,i1,a4)') 'A3',mype,'.mfn'
      endif
      if(mype>9)then
      write(mfnfile,'(a2,i2,a4)') 'A3',mype,'.mfn'
      endif

C Change the .wel file before calling MODFLOW
      call writewel(DIM/3,X,Y,Z,WELRATE,mype)
      CLOSE(13)


C Call MODFLOW
      call system("echo "//mfnfile//" | modflw96")

      

C Read in the heads from the .hed file that MODFLOW outputs
      call readhead(NROW,NCOL,NLAY,HEAD,mype)
      DO 10 , K=1,NLAY
C      WRITE(*,*)  ((HEAD(I,J,K),J=1,NCOL),I=1,NROW)
 10   CONTINUE


C Compute the installation cost
      call costac(DIM/3,X,Y,Z,WELRATE,NLAY,NROW,NCOL,COSTC,FLAG)
      WRITE(*,*) 'costc', COSTC

C Compute the operational cost
      call costao(DIM/3,X,Y,Z,WELRATE,HEAD,NLAY,NROW,NCOL,COSTO,FLAG)
      WRITE(*,*) 'costo', COSTO
      FV = COSTC + COSTO

C Check to see if the constraints are violated
      call head_constraint(DIM/3,X,Y,Z,WELRATE,HEAD,NLAY,NROW,
     & NCOL,COSTO,FLAG)

      WRITE(*,*) '********************************'
      WRITE(*,100) ' Function value =', FV
      WRITE(*,*) 'FLAG=', FLAG
      WRITE(*,*) '********************************'

 100   FORMAT(a17,E13.5)

      ENDIF

      END



