/*  _________________________________________________________________________
 *
 *  UTILIB: A utility library for developing portable C++ codes.
 *  Copyright (c) 2008 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the README file in the top UTILIB directory.
 *  _________________________________________________________________________
 */

/** WEH - This code has been modified to pass SIGTERM to the child process. 
	I also changed the format to look like what you get from
		/usr/bin/time --portability
 */

/*
 * Copyright (c) 1987, 1988, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] =
"@(#) Copyright (c) 1987, 1988, 1993\n\
	The Regents of the University of California.  All rights reserved.\n";
#endif /* not lint */

#ifndef lint
#if 0
static char sccsid[] = "@(#)time.c	8.1 (Berkeley) 6/6/93";
#endif
static const char rcsid[] =
  "$FreeBSD: src/usr.bin/time/time.c,v 1.30 2007/05/07 12:23:23 dwmalone Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <sys/resource.h>
#include <sys/signal.h>

//#include <sys/sysctl.h>
#include <sys/time.h>
#include <sys/wait.h>

//#include <err.h>
#include <errno.h>
#include <locale.h>

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>

#include <utilib_config.h>
#ifdef UTILIB_HAVE_ERR_H
//#include <sys/sysctl.h>
#include <err.h>
#else
// Solaris & AIX don't support err.h
void err(int eval, const char* fmt, const char* msg)
{
        fprintf( stderr, "ERROR:" );
	if ( msg )
	   fprintf( stderr, fmt, msg );
	else
	   fprintf( stderr, fmt );
        if( eval ) exit(eval);
} 
void
warn(const char* fmt) 
{
   fprintf( stderr, "WARNING" );
   fprintf( stderr, fmt );
}
void
warnx(const char* fmt ) 
{ warn(fmt); }
#endif

/*static int getstathz(void);*/
static void humantime(FILE *, long, long);
static void showtime(FILE *, struct timeval *, struct timeval *,
    struct rusage *);
/*static void siginfo(int);*/
static void usage(void);

pid_t pid;

void kill_child(int signum)
{
kill(pid,signum);
}

static char decimal_point;
static struct timeval before_tv;
static int hflag, pflag;

int
main(int argc, char **argv)
{
        extern char *optarg;
        extern int optind;
	int aflag, ch, lflag, status;
	int exitonsig;
	struct rlimit rl;
	struct rusage ru;
	struct timeval after;
	char *ofn = NULL;
	FILE *out = stdout;

	(void) setlocale(LC_NUMERIC, "");
	decimal_point = localeconv()->decimal_point[0];

	aflag = hflag = lflag = pflag = 0;
	while ((ch = getopt(argc, argv, "+aho:p")) != -1)
		switch((char)ch) {
		case 'a':
			aflag = 1;
			break;
		case 'h':
			hflag = 1;
			break;
		case 'o':
			ofn = optarg;
			break;
		case 'p':
			pflag = 1;
			break;
		case '?':
		default:
			usage();
		}

	if (!(argc -= optind)) {
        usage();
		exit(0);
        }
	argv += optind;

	if (ofn) {
	        if ((out = fopen(ofn, aflag ? "a" : "w")) == NULL)
		        err(1, "%s", ofn);
		setvbuf(out, (char *)NULL, _IONBF, (size_t)0);
	}

	gettimeofday(&before_tv, (struct timezone *)NULL);
	switch(pid = fork()) {
	case -1:			/* error */
	   err(1, "%s", "time");
		/* NOTREACHED */
	case 0:				/* child */
		if (ofn)
			fclose(out);
		execvp(*argv, argv);
		err(errno == ENOENT ? 127 : 126, "%s", *argv);
		/* NOTREACHED */
	}
	/* parent */
	(void)signal(SIGINT, kill_child);
	(void)signal(SIGQUIT, kill_child);
	(void)signal(SIGTERM, kill_child);
	/*(void)signal(SIGINFO, siginfo);*/
	while (wait4(pid, &status, 0, &ru) != pid);
	gettimeofday(&after, (struct timezone *)NULL);
	if ( ! WIFEXITED(status))
		warnx("command terminated abnormally");
	exitonsig = WIFSIGNALED(status) ? WTERMSIG(status) : 0;
	showtime(out, &before_tv, &after, &ru);
#if 0
	if (lflag) {
		int hz = getstathz();
		u_long ticks;

		ticks = hz * (ru.ru_utime.tv_sec + ru.ru_stime.tv_sec) +
		     hz * (ru.ru_utime.tv_usec + ru.ru_stime.tv_usec) / 1000000;

		/*
		 * If our round-off on the tick calculation still puts us at 0,
		 * then always assume at least one tick.
		 */
		if (ticks == 0)
			ticks = 1;

		fprintf(out, "%10ld  %s\n",
			ru.ru_maxrss, "maximum resident set size");
		fprintf(out, "%10ld  %s\n",
			ru.ru_ixrss / ticks, "average shared memory size");
		fprintf(out, "%10ld  %s\n",
			ru.ru_idrss / ticks, "average unshared data size");
		fprintf(out, "%10ld  %s\n",
			ru.ru_isrss / ticks, "average unshared stack size");
		fprintf(out, "%10ld  %s\n",
			ru.ru_minflt, "page reclaims");
		fprintf(out, "%10ld  %s\n",
			ru.ru_majflt, "page faults");
		fprintf(out, "%10ld  %s\n",
			ru.ru_nswap, "swaps");
		fprintf(out, "%10ld  %s\n",
			ru.ru_inblock, "block input operations");
		fprintf(out, "%10ld  %s\n",
			ru.ru_oublock, "block output operations");
		fprintf(out, "%10ld  %s\n",
			ru.ru_msgsnd, "messages sent");
		fprintf(out, "%10ld  %s\n",
			ru.ru_msgrcv, "messages received");
		fprintf(out, "%10ld  %s\n",
			ru.ru_nsignals, "signals received");
		fprintf(out, "%10ld  %s\n",
			ru.ru_nvcsw, "voluntary context switches");
		fprintf(out, "%10ld  %s\n",
			ru.ru_nivcsw, "involuntary context switches");
	}
#endif
	/*
	 * If the child has exited on a signal, exit on the same
	 * signal, too, in order to reproduce the child's exit status.
	 * However, avoid actually dumping core from the current process.
	 */
	if (exitonsig) {
		if (signal(exitonsig, SIG_DFL) == SIG_ERR)
			warn("signal");
		else {
			rl.rlim_max = rl.rlim_cur = 0;
			if (setrlimit(RLIMIT_CORE, &rl) == -1)
				warn("setrlimit");
			kill(getpid(), exitonsig);
		}
	}
	exit (WIFEXITED(status) ? WEXITSTATUS(status) : EXIT_FAILURE);
}

static void
usage(void)
{
	fprintf(stderr, "usage: timer [-a] [-h | -p] [-o file] utility [argument ...]\n\n");
	fprintf(stderr, "An adaptation of the standard 'time' utility that passes SIGTERM to\n  the child process.\n\n");
    fprintf(stderr, "Options:\n");
    fprintf(stderr, "  -a                     When the -o option is specified, append timing data\n");
    fprintf(stderr, "  -h                     Convert time in days, minutes and seconds\n");
    fprintf(stderr, "  --help                 Print this help information\n");
    fprintf(stderr, "  -o FILE                Send timing information to a file\n");
    fprintf(stderr, "  -p                     Print the timing data in the Posix format\n");
	exit(1);
}

#if 0
/*
 * Return the frequency of the kernel's statistics clock.
 */
static int
getstathz(void)
{
	int mib[2];
	size_t size;
	struct clockinfo clockrate;

	mib[0] = CTL_KERN;
	mib[1] = KERN_CLOCKRATE;
	size = sizeof clockrate;
	if (sysctl(mib, 2, &clockrate, &size, NULL, 0) == -1)
	   err(1, "%s", "sysctl kern.clockrate");
	return clockrate.stathz;
}
#endif

static void
humantime(FILE *out, long sec, long usec)
{
	long days, hrs, mins;

	days = sec / (60L * 60 * 24);
	sec %= (60L * 60 * 24);
	hrs = sec / (60L * 60);
	sec %= (60L * 60);
	mins = sec / 60;
	sec %= 60;

	fprintf(out, "\t");
	if (days)
		fprintf(out, "%ldd", days);
	if (hrs)
		fprintf(out, "%ldh", hrs);
	if (mins)
		fprintf(out, "%ldm", mins);
	fprintf(out, "%ld%c%02lds", sec, decimal_point, usec);
}

static void
showtime(FILE *out, struct timeval *before, struct timeval *after,
    struct rusage *ru)
{

	after->tv_sec -= before->tv_sec;
	after->tv_usec -= before->tv_usec;
	if (after->tv_usec < 0)
		after->tv_sec--, after->tv_usec += 1000000;

	if (pflag) {
		/* POSIX wants output that must look like
		"real %f\nuser %f\nsys %f\n" and requires
		at least two digits after the radix. */
		fprintf(out, "real %jd%c%02ld\n",
			(intmax_t)after->tv_sec, decimal_point,
			(long int)after->tv_usec/10000);
		fprintf(out, "user %jd%c%02ld\n",
			(intmax_t)ru->ru_utime.tv_sec, decimal_point,
			(long int)ru->ru_utime.tv_usec/10000);
		fprintf(out, "sys %jd%c%02ld\n",
			(intmax_t)ru->ru_stime.tv_sec, decimal_point,
			(long int)ru->ru_stime.tv_usec/10000);
	} else if (hflag) {
		fprintf(out, "real\t");
		humantime(out, after->tv_sec, after->tv_usec/10000);
		fprintf(out, "\nuser\t");
		humantime(out, ru->ru_utime.tv_sec, ru->ru_utime.tv_usec/10000);
		fprintf(out, "\nsys\t");
		humantime(out, ru->ru_stime.tv_sec, ru->ru_stime.tv_usec/10000);
		fprintf(out, "\n");
	} else {
		fprintf(out, "real\t%9jd%c%02ld\n",
			(intmax_t)after->tv_sec, decimal_point,
			(long int)after->tv_usec/10000);
		fprintf(out, "user\t%9jd%c%02ld\n",
			(intmax_t)ru->ru_utime.tv_sec, decimal_point,
			(long int)ru->ru_utime.tv_usec/10000);
		fprintf(out, "sys\t%9jd%c%02ld\n",
			(intmax_t)ru->ru_stime.tv_sec, decimal_point,
			(long int)ru->ru_stime.tv_usec/10000);
	}
}

#if 0
static void
siginfo(int sig __unused)
{
	struct timeval after;
	struct rusage ru;

	gettimeofday(&after, (struct timezone *)NULL);
	getrusage(RUSAGE_CHILDREN, &ru);
	showtime(stdout, &before_tv, &after, &ru);
}
#endif
