//-----------------------------------------------------------------------bl-
//--------------------------------------------------------------------------
//
// QUESO - a library to support the Quantification of Uncertainty
// for Estimation, Simulation and Optimization
//
// Copyright (C) 2008-2017 The PECOS Development Team
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the Version 2.1 GNU Lesser General
// Public License as published by the Free Software Foundation.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc. 51 Franklin Street, Fifth Floor,
// Boston, MA  02110-1301  USA
//
//-----------------------------------------------------------------------el-

#include "config_queso.h"

#ifdef QUESO_HAVE_CPPUNIT

#ifdef QUESO_HAVE_BOOST

#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestCase.h>

#include <queso/Environment.h>
#include <queso/RngBoost.h>

namespace QUESOTesting
{

class RngBoost : public CppUnit::TestCase
{
public:
  CPPUNIT_TEST_SUITE(RngBoost);
  CPPUNIT_TEST(test_beta);
  CPPUNIT_TEST(test_gamma);
  CPPUNIT_TEST(test_uniform);
  CPPUNIT_TEST(test_gaussian);
  CPPUNIT_TEST_SUITE_END();

  // yes, this is necessary
public:
  void setUp()
  {
    _env.reset(new QUESO::FullEnvironment("","",NULL));
  }

  void test_beta()
  {
    QUESO::RngBoost rng_boost(0, 0);
    rng_boost.resetSeed(0);

    double mean = 0.0;
    unsigned int num_samples = 1000000;

    for (unsigned int i = 0; i < num_samples; i++) {
      mean += rng_boost.betaSample(2.0, 2.0) / num_samples;
    }

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, mean, 1e-2);
  }

  void test_gamma()
  {
    QUESO::RngBoost rng_boost(0, 0);

    double mean = 0.0;
    unsigned int num_samples = 1000000;

    for (unsigned int i = 0; i < num_samples; i++) {
      mean += rng_boost.gammaSample(10.0, 0.5) / num_samples;
    }

    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.0, mean, 1e-2);
  }

  void test_uniform()
  {
    QUESO::RngBoost rng_boost(0, 0);

    double mean = 0.0;
    unsigned int num_samples = 1000000;

    for (unsigned int i = 0; i < num_samples; i++) {
      mean += rng_boost.uniformSample() / num_samples;
    }

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, mean, 1e-2);
  }

  void test_gaussian()
  {
    QUESO::RngBoost rng_boost(0, 0);

    double mean = 0.0;
    unsigned int num_samples = 1000000;

    for (unsigned int i = 0; i < num_samples; i++) {
      mean += rng_boost.gaussianSample(0.1) / num_samples;
    }

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, mean, 1e-2);
  }

private:
  typename QUESO::ScopedPtr<QUESO::BaseEnvironment>::Type _env;
};

CPPUNIT_TEST_SUITE_REGISTRATION(RngBoost);

} // end namespace QUESOTesting

#endif  // QUESO_HAVE_BOOST

#endif // QUESO_HAVE_CPPUNIT
